@@ -30,7 +30,6 @@ from barman.clients.cloud_cli import (
     NetworkErrorExit,
     OperationErrorExit,
     UrlArgumentType,
-    get_missing_attrs,
 )
 from barman.cloud import (
     CloudBackupSnapshot,
@@ -116,19 +115,11 @@ def _validate_config(config):
     required_snapshot_variables = (
         "snapshot_disks",
         "snapshot_instance",
-        "snapshot_zone",
     )
     is_snapshot_backup = any(
         [getattr(config, var) for var in required_snapshot_variables]
     )
     if is_snapshot_backup:
-        missing_options = get_missing_attrs(config, required_snapshot_variables)
-        if len(missing_options) > 0:
-            raise ConfigurationException(
-                "Incomplete options for snapshot backup - missing: %s"
-                % ", ".join(missing_options)
-            )
-
         if getattr(config, "compression"):
             raise ConfigurationException(
                 "Compression options cannot be used with snapshot backups"
@@ -209,15 +200,15 @@ def main(args=None):
 
                 with closing(postgres):
                     # Take snapshot backups if snapshot backups were specified
-                    if config.snapshot_instance:
+                    if config.snapshot_disks or config.snapshot_instance:
                         snapshot_interface = get_snapshot_interface(config)
+                        snapshot_interface.validate_backup_config(config)
                         snapshot_backup = CloudBackupSnapshot(
                             config.server_name,
                             cloud_interface,
                             snapshot_interface,
                             postgres,
                             config.snapshot_instance,
-                            config.snapshot_zone,
                             config.snapshot_disks,
                             config.backup_name,
                         )
@@ -351,13 +342,25 @@ def parse_arguments(args=None):
     )
     parser.add_argument(
         "--snapshot-zone",
-        help="Zone of the disks from which snapshots should be taken",
+        help=(
+            "Zone of the disks from which snapshots should be taken (deprecated: "
+            "replaced by --gcp-zone)"
+        ),
+        dest="gcp_zone",
     )
     gcs_arguments = parser.add_argument_group(
         "Extra options for google-cloud-storage cloud provider"
     )
     gcs_arguments.add_argument(
         "--snapshot-gcp-project",
+        help=(
+            "GCP project under which disk snapshots should be stored (deprecated: "
+            "replaced by --gcp-project)"
+        ),
+        dest="gcp_project",
+    )
+    gcs_arguments.add_argument(
+        "--gcp-project",
         help="GCP project under which disk snapshots should be stored",
     )
     gcs_arguments.add_argument(
@@ -365,6 +368,10 @@ def parse_arguments(args=None):
         help="The name of the GCP KMS key which should be used for encrypting the "
         "uploaded data in GCS.",
     )
+    gcs_arguments.add_argument(
+        "--gcp-zone",
+        help="Zone of the disks from which snapshots should be taken",
+    )
     add_tag_argument(
         parser,
         name="tags",
@@ -388,6 +395,16 @@ def parse_arguments(args=None):
         help="The name of an encryption scope defined in the Azure Blob Storage "
         "service which is to be used to encrypt the data in Azure",
     )
+    azure_arguments.add_argument(
+        "--azure-subscription-id",
+        help="The ID of the Azure subscription which owns the instance and storage "
+        "volumes defined by the --snapshot-instance and --snapshot-disk arguments.",
+    )
+    azure_arguments.add_argument(
+        "--azure-resource-group",
+        help="The name of the Azure resource group to which the compute instance and "
+        "disks defined by the --snapshot-instance and --snapshot-disk arguments belong.",
+    )
     return parser.parse_args(args=args)
 
 
