@@ -26,7 +26,6 @@ from barman.clients.cloud_cli import (
     GeneralErrorExit,
     NetworkErrorExit,
     OperationErrorExit,
-    get_missing_attrs,
 )
 from barman.cloud import CloudBackupCatalog, configure_logging
 from barman.cloud_providers import (
@@ -49,14 +48,6 @@ def _validate_config(config, backup_info):
     :param BackupInfo backup_info: The backup info for the backup to restore
     """
     if backup_info.snapshots_info:
-        missing_options = get_missing_attrs(
-            config, ("snapshot_recovery_instance", "snapshot_recovery_zone")
-        )
-        if len(missing_options) > 0:
-            raise ConfigurationException(
-                "Incomplete options for snapshot restore - missing: %s"
-                % ", ".join(missing_options)
-            )
         if config.tablespace != []:
             raise ConfigurationException(
                 "Backup %s is a snapshot backup therefore tablespace relocation rules "
@@ -103,12 +94,17 @@ def main(args=None):
             _validate_config(config, backup_info)
 
             if backup_info.snapshots_info:
-                downloader = CloudBackupDownloaderSnapshot(cloud_interface, catalog)
+                snapshot_interface = get_snapshot_interface_from_backup_info(
+                    backup_info, config
+                )
+                snapshot_interface.validate_restore_config(config)
+                downloader = CloudBackupDownloaderSnapshot(
+                    cloud_interface, catalog, snapshot_interface
+                )
                 downloader.download_backup(
                     backup_info,
                     config.recovery_dir,
                     config.snapshot_recovery_instance,
-                    config.snapshot_recovery_zone,
                 )
             else:
                 downloader = CloudBackupDownloaderObjectStore(cloud_interface, catalog)
@@ -135,7 +131,7 @@ def parse_arguments(args=None):
     :return: The options parsed
     """
 
-    parser, _, _ = create_argument_parser(
+    parser, _, azure_arguments = create_argument_parser(
         description="This script can be used to download a backup "
         "previously made with barman-cloud-backup command."
         "Currently AWS S3, Azure Blob Storage and Google Cloud Storage are supported.",
@@ -155,8 +151,23 @@ def parse_arguments(args=None):
     )
     parser.add_argument(
         "--snapshot-recovery-zone",
+        help=(
+            "Zone containing the instance and disks for the snapshot recovery "
+            "(deprecated: replaced by --gcp-zone)"
+        ),
+        dest="gcp_zone",
+    )
+    gcs_arguments = parser.add_argument_group(
+        "Extra options for google-cloud-storage cloud provider"
+    )
+    gcs_arguments.add_argument(
+        "--gcp-zone",
         help="Zone containing the instance and disks for the snapshot recovery",
     )
+    azure_arguments.add_argument(
+        "--azure-resource-group",
+        help="Resource group containing the instance and disks for the snapshot recovery",
+    )
     return parser.parse_args(args=args)
 
 
@@ -303,8 +314,25 @@ class CloudBackupDownloaderObjectStore(CloudBackupDownloader):
 class CloudBackupDownloaderSnapshot(CloudBackupDownloader):
     """A minimal downloader for cloud backups which just retrieves the backup label."""
 
+    def __init__(self, cloud_interface, catalog, snapshot_interface):
+        """
+        Object responsible for handling interactions with cloud storage
+
+        :param CloudInterface cloud_interface: The interface to use to
+          upload the backup
+        :param str server_name: The name of the server as configured in Barman
+        :param CloudBackupCatalog catalog: The cloud backup catalog
+        :param CloudSnapshotInterface snapshot_interface: Interface for managing
+            snapshots via a cloud provider API.
+        """
+        super(CloudBackupDownloaderSnapshot, self).__init__(cloud_interface, catalog)
+        self.snapshot_interface = snapshot_interface
+
     def download_backup(
-        self, backup_info, destination_dir, recovery_instance, recovery_zone
+        self,
+        backup_info,
+        destination_dir,
+        recovery_instance,
     ):
         """
         Download a backup from cloud storage
@@ -313,17 +341,14 @@ class CloudBackupDownloaderSnapshot(CloudBackupDownloader):
         :param str destination_dir: Path to the destination directory
         :param str recovery_instance: The name of the VM instance to which the disks
             cloned from the backup snapshots are attached.
-        :param str recovery_zone: The zone in which the recovery disks and instance
-            reside.
         """
-        snapshot_interface = get_snapshot_interface_from_backup_info(backup_info)
-        attached_snapshots = SnapshotRecoveryExecutor.get_attached_snapshots_for_backup(
-            snapshot_interface, backup_info, recovery_instance, recovery_zone
+        attached_volumes = SnapshotRecoveryExecutor.get_attached_volumes_for_backup(
+            self.snapshot_interface,
+            backup_info,
+            recovery_instance,
         )
         cmd = UnixLocalCommand()
-        SnapshotRecoveryExecutor.check_mount_points(
-            backup_info, attached_snapshots, cmd
-        )
+        SnapshotRecoveryExecutor.check_mount_points(backup_info, attached_volumes, cmd)
         SnapshotRecoveryExecutor.check_recovery_dir_exists(destination_dir, cmd)
 
         # If the target directory does not exist then we will fail here because
