@@ -429,6 +429,9 @@ class ServerConfig(object):
         "active",
         "archiver",
         "archiver_batch_size",
+        "azure_credential",
+        "azure_resource_group",
+        "azure_subscription_id",
         "backup_compression",
         "backup_compression_format",
         "backup_compression_level",
@@ -451,6 +454,8 @@ class ServerConfig(object):
         "disabled",
         "errors_directory",
         "forward_config_path",
+        "gcp_project",
+        "gcp_zone",
         "immediate_checkpoint",
         "incoming_wals_directory",
         "last_backup_maximum_age",
@@ -493,10 +498,10 @@ class ServerConfig(object):
         "reuse_backup",
         "slot_name",
         "snapshot_disks",
-        "snapshot_gcp_project",
+        "snapshot_gcp_project",  # Deprecated, replaced by gcp_project
         "snapshot_instance",
         "snapshot_provider",
-        "snapshot_zone",
+        "snapshot_zone",  # Deprecated, replaced by gcp_zone
         "ssh_command",
         "streaming_archiver",
         "streaming_archiver_batch_size",
@@ -512,6 +517,9 @@ class ServerConfig(object):
     BARMAN_KEYS = [
         "archiver",
         "archiver_batch_size",
+        "azure_credential",
+        "azure_resource_group",
+        "azure_subscription_id",
         "backup_compression",
         "backup_compression_format",
         "backup_compression_level",
@@ -529,6 +537,7 @@ class ServerConfig(object):
         "custom_decompression_filter",
         "custom_compression_magic",
         "forward_config_path",
+        "gcp_project",
         "immediate_checkpoint",
         "last_backup_maximum_age",
         "last_backup_minimum_size",
@@ -568,7 +577,7 @@ class ServerConfig(object):
         "retention_policy_mode",
         "reuse_backup",
         "slot_name",
-        "snapshot_gcp_project",
+        "snapshot_gcp_project",  # Deprecated, replaced by gcp_project
         "snapshot_provider",
         "streaming_archiver",
         "streaming_archiver_batch_size",
